/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	Devices vector table for GCC based toolchain:
	STM32H723/25
	STM32H730/733/735
	STM32H742/743
	STM32H745/747
	STM32H750/753
	STM32H755/757
	STM32H7Ax
	STM32H7Bx

	Exception vectors and default handlers for Cortex-M3, Cortex-M4, Cortex-M7
	Interrupts vector and default handlers

	After Reset the Cortex-M processor is in Thread mode,
	priority is Privileged, and the stack is set to Main.

	When		Who	What
	13/10/18	ac	Creation
	17/03/20	ac	Add H742, H750
	30/01/22	ac	Remove old H7, add devices from STM32Cube_FW_H7_V1.9.1 06-Dec_2021

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/
/*
	__ARM_ARCH_6M__		Set to 1 when generating code for Armv6-M (Cortex-M0, Cortex-M0+, Cortex-M1)
	__ARM_ARCH_7M__		Set to 1 when generating code for Armv7-M (Cortex-M3)
	__ARM_ARCH_7EM__	Set to 1 when generating code for Armv7-M (Cortex-M4, Cortex-M7) with FPU
*/

// One of STM32H742xx  STM32H743xx  STM32H753xx  STM32H750xx must be defined
#include	<stm32h7xx.h>

#if (defined __CC_ARM)
	static inline void __DEBUG_BKPT (void)
	{
		if ((CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk) != 0u)
		{
			// Debug enabled, can use BKPT instruction
			__breakpoint (0) ;
		}
		else
		{
			while (1) { }
		}
	}
#else
	__attribute__ ((always_inline)) static inline void __DEBUG_BKPT (void)
	{
		if ((CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk) != 0u)
		{
			// Debug enabled, can use BKPT instruction
			__ASM volatile ("bkpt 0") ;
		}
		else
		{
			while (1) { }
		}
	}
#endif

extern void __attribute__((noreturn)) start_ (void);	// The entry point in startup.c

// ----------------------------------------------------------------------------
// Exception Stack Frame of the Cortex-M0-M0+-M3-M4-M7 processors.

typedef struct
{
	uint32_t	r0 ;
	uint32_t	r1 ;
	uint32_t	r2 ;
	uint32_t	r3 ;
	uint32_t	r12 ;
	uint32_t	lr ;
	uint32_t	pc ;
	uint32_t	psr ;
	#if  defined(__ARM_ARCH_7EM__)
		uint32_t s[16];				// Floating point registers
	#endif

} exceptionStackFrame ;

void	  HardFault_Handler_C (exceptionStackFrame * frame, uint32_t lr) ;

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

	void	  UsageFault_Handler_C	(exceptionStackFrame * frame, uint32_t lr) ;
	void	  BusFault_Handler_C	(exceptionStackFrame * frame, uint32_t lr) ;

#endif // defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

// ----------------------------------------------------------------------------
// Forward declarations of exception handlers
// There is specific default handlers for these exception at the end of the file

extern void		Reset_Handler		(void) ;
extern void		NMI_Handler			(void) ;
extern void		HardFault_Handler	(void) ;

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)
	extern void	MemManage_Handler	(void) ;
	extern void	BusFault_Handler	(void) ;
	extern void	UsageFault_Handler	(void) ;
	extern void	DebugMon_Handler	(void) ;
#endif

extern void		SVC_Handler			(void) ;
extern void		PendSV_Handler		(void) ;
extern void		SysTick_Handler		(void) ;

// Generic default handler for interruptions
void __attribute__((weak)) Default_Handler (void) ;

// ----------------------------------------------------------------------------
// Forward declaration of the IRQ handlers with weak definitions.
// The application can defines a handler with the same name and take precedence

void __attribute__ ((weak, alias ("Default_Handler")))	WWDG_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	PVD_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TAMP_STAMP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	RTC_WKUP_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FLASH_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	RCC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI0_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream2_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream3_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream4_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream5_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream6_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	ADC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN1_IT0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN2_IT0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN1_IT1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN2_IT1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI9_5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM1_BRK_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM1_UP_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM1_TRG_COM_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM1_CC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM2_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM3_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM4_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C1_EV_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C1_ER_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C2_EV_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C2_ER_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPI1_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPI2_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	USART1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	USART2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	USART3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	EXTI15_10_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	RTC_Alarm_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM8_BRK_TIM12_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM8_UP_TIM13_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM8_TRG_COM_TIM14_IRQHandler	(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM8_CC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA1_Stream7_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FMC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SDMMC1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM5_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPI3_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	UART4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	UART5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM6_DAC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM7_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream2_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream3_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream4_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	ETH_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	ETH_WKUP_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN_CAL_IRQHandler 			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	CM7_SEV_IRQHandler 				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	CM4_SEV_IRQHandler 				(void) ;
//void __attribute__ ((weak, alias ("Default_Handler")))	CAN2_SCE_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream5_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream6_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2_Stream7_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	USART6_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C3_EV_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C3_ER_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_HS_EP1_OUT_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_HS_EP1_IN_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_HS_WKUP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_HS_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DCMI_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	CRYP_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HASH_RNG_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FPU_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	UART7_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	UART8_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPI4_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPI5_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPI6_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SAI1_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LTDC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LTDC_ER_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	JPEG_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMA2D_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SAI2_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	QUADSPI_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LPTIM1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	CEC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C4_EV_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C4_ER_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SPDIF_RX_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_FS_EP1_OUT_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_FS_EP1_IN_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_FS_WKUP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTG_FS_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMAMUX1_OVR_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_Master_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_TIMA_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_TIMB_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_TIMC_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_TIMD_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_TIME_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HRTIM1_FLT_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT2_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT3_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT4_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT5_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT6_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DFSDM1_FLT7_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SAI3_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SWPMI1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM15_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM16_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM17_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	MDIOS_WKUP_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	MDIOS_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	MDMA_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DSI_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SDMMC2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HSEM1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HSEM2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	ADC3_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DAC2_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DMAMUX2_OVR_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel0_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel1_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel2_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel3_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel4_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel5_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel6_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA_Channel7_IRQHandler		(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	COMP1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LPTIM2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LPTIM3_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LPTIM4_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LPTIM5_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	LPUART1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	WWDG_RST_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	CRS_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	ECC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	SAI4_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	HOLD_CORE_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	DTS_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	WAKEUP_PIN_IRQHandler			(void) ;

void __attribute__ ((weak, alias ("Default_Handler")))	OCTOSPI2_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTFDEC1_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	OTFDEC2_IRQHandler				(void) ;

void __attribute__ ((weak, alias ("Default_Handler")))	FMAC_IRQHandler					(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	CORDIC_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	GFXMMU_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	BDMA1_IRQHandler				(void) ;

void __attribute__ ((weak, alias ("Default_Handler")))	UART9_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	USART10_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C5_EV_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	I2C5_ER_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN3_IT0_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	FDCAN3_IT1_IRQHandler			(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM23_IRQHandler				(void) ;
void __attribute__ ((weak, alias ("Default_Handler")))	TIM24_IRQHandler				(void) ;

// ----------------------------------------------------------------------------

extern unsigned int _estack_ ;				// From linker script: initial stack address

typedef void (* const pHandler) (void) ;	// Type of the interruptions vector array

// ----------------------------------------------------------------------------

// The table of interrupt handlers. It has an explicit section name
// and relies on the linker script to place it at the correct location in memory.

// Can't place integer and function pointer in one array (Forbidden by ISO C)
__attribute__ ((section(".isr_stackaddress"),used)) unsigned int * isrStackAdress =
{
	& _estack_				// From linker script: initial Main stack address
} ;

__attribute__ ((section(".isr_vectors"),used)) pHandler isrVectors_[] =
{
	// Cortex-M Core Handlers
	//									   The initial Main stack pointer is in the section .isr_stackaddress
	Reset_Handler,						// The reset handler

	NMI_Handler,						// The NMI handler
	HardFault_Handler,					// The hard fault handler
	MemManage_Handler,					// The MPU fault handler
	BusFault_Handler,					// The bus fault handler
	UsageFault_Handler,					// The usage fault handler
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
	0,									// Reserved
	SVC_Handler,						// SVCall handler
	DebugMon_Handler,					// Debug monitor handler
	0,									// Reserved
	PendSV_Handler,						// The PendSV handler
	SysTick_Handler,					// The SysTick handler

	// ----------------------------------------------------------------------
	// External Interrupts

	WWDG_IRQHandler,					// 0	Window WatchDog Interrupt ( wwdg1_it, wwdg2_it)
	PVD_IRQHandler,						// 1	PVD through EXTI Line detection
	TAMP_STAMP_IRQHandler,				// 2	Tamper and TimeStamps through the EXTI line
	RTC_WKUP_IRQHandler,				// 3	RTC Wakeup interrupt through the EXTI line
	FLASH_IRQHandler,					// 4	FLASH global Interrupt
	RCC_IRQHandler,						// 5	RCC global Interrupt
	EXTI0_IRQHandler,					// 6	EXTI Line0 Interrupt
	EXTI1_IRQHandler,					// 7	EXTI Line1 Interrupt
	EXTI2_IRQHandler,					// 8	EXTI Line2 Interrupt
	EXTI3_IRQHandler,					// 9	EXTI Line3 Interrupt
	EXTI4_IRQHandler,					// 10	EXTI Line4 Interrupt
	DMA1_Stream0_IRQHandler,			// 11	DMA1 Stream 0 global Interrupt
	DMA1_Stream1_IRQHandler,			// 12	DMA1 Stream 1 global Interrupt
	DMA1_Stream2_IRQHandler,			// 13	DMA1 Stream 2 global Interrupt
	DMA1_Stream3_IRQHandler,			// 14	DMA1 Stream 3 global Interrupt
	DMA1_Stream4_IRQHandler,			// 15	DMA1 Stream 4 global Interrupt
	DMA1_Stream5_IRQHandler,			// 16	DMA1 Stream 5 global Interrupt
	DMA1_Stream6_IRQHandler,			// 17	DMA1 Stream 6 global Interrupt
	ADC_IRQHandler,						// 18	ADC1, ADC2 and ADC3s
	FDCAN1_IT0_IRQHandler,				// 19	FDCAN1 Interrupt line 0
	FDCAN2_IT0_IRQHandler,				// 20	FDCAN2 Interrupt line 0
	FDCAN1_IT1_IRQHandler,				// 21	FDCAN1 Interrupt line 1
	FDCAN2_IT1_IRQHandler,				// 22	FDCAN2 Interrupt line 1
	EXTI9_5_IRQHandler,					// 23	External Line[9:5] Interrupts
	TIM1_BRK_IRQHandler,				// 24	TIM1 Break Interrupt
	TIM1_UP_IRQHandler,					// 25	TIM1 Update Interrupt
	TIM1_TRG_COM_IRQHandler,			// 26	TIM1 Trigger and Commutation Interrupt
	TIM1_CC_IRQHandler,					// 27	TIM1 Capture Compare Interrupt
	TIM2_IRQHandler,					// 28	TIM2 global Interrupt
	TIM3_IRQHandler,					// 29	TIM3 global Interrupt
	TIM4_IRQHandler,					// 30	TIM4 global Interrupt
	I2C1_EV_IRQHandler,					// 21	I2C1 Event Interrupt
	I2C1_ER_IRQHandler,					// 32	I2C1 Error Interrupt
	I2C2_EV_IRQHandler,					// 33	I2C2 Event Interrupt
	I2C2_ER_IRQHandler,					// 34	I2C2 Error Interrupt
	SPI1_IRQHandler,					// 35	SPI1 global Interrupt
	SPI2_IRQHandler,					// 36	SPI2 global Interrupt
	USART1_IRQHandler,					// 37	USART1 global Interrupt
	USART2_IRQHandler,					// 38	USART2 global Interrupt
	USART3_IRQHandler,					// 39	USART3 global Interrupt
	EXTI15_10_IRQHandler,				// 40	External Line[15:10] Interrupts
	RTC_Alarm_IRQHandler,				// 41	RTC Alarm (A and B) through EXTI Line Interrupt
	DFSDM2_IRQHandler,					// 42	DFSDM2 Interrupt
	TIM8_BRK_TIM12_IRQHandler,			// 43	TIM8 Break Interrupt and TIM12 global interrupt
	TIM8_UP_TIM13_IRQHandler,			// 44	TIM8 Update Interrupt and TIM13 global interrupt
	TIM8_TRG_COM_TIM14_IRQHandler,		// 45	TIM8 Trigger and Commutation Interrupt and TIM14global interrupt
	TIM8_CC_IRQHandler,					// 46	TIM8 Capture Compare Interrupt
	DMA1_Stream7_IRQHandler,			// 47	DMA1 Stream7 Interrupt
	FMC_IRQHandler,						// 48	FMC global Interrupt
	SDMMC1_IRQHandler,					// 49	SDMMC1 global Interrupt
	TIM5_IRQHandler,					// 50	TIM5 global Interrupt
	SPI3_IRQHandler,					// 51	SPI3 global Interrupt
	UART4_IRQHandler,					// 52	UART4 global Interrupt
	UART5_IRQHandler,					// 53	UART5 global Interrupt
	TIM6_DAC_IRQHandler,				// 54	TIM6 global and DAC1&2 underrun error  interrupts
	TIM7_IRQHandler,					// 55	TIM7 global interrupt
	DMA2_Stream0_IRQHandler,			// 56	DMA2 Stream 0 global Interrupt
	DMA2_Stream1_IRQHandler,			// 57	DMA2 Stream 1 global Interrupt
	DMA2_Stream2_IRQHandler,			// 58	DMA2 Stream 2 global Interrupt
	DMA2_Stream3_IRQHandler,			// 59	DMA2 Stream 3 global Interrupt
	DMA2_Stream4_IRQHandler,			// 60	DMA2 Stream 4 global Interrupt
	ETH_IRQHandler,						// 61	Ethernet
	ETH_WKUP_IRQHandler,				// 62	Ethernet Wakeup through EXTI line
	FDCAN_CAL_IRQHandler,				// 63	FDCAN calibration unit interrupt
#if (defined DFSDM1_Filter4)
	DFSDM1_FLT4_IRQHandler,				// 64	DFSDM Filter4 Interrup
	DFSDM1_FLT5_IRQHandler,				// 65	DFSDM Filter5 Interrup
	DFSDM1_FLT6_IRQHandler,				// 66	DFSDM Filter6 InterrupE
	DFSDM1_FLT7_IRQHandler,				// 67	DFSDM Filter7 Interrup
#else
	CM7_SEV_IRQHandler,					// 64	CM7 Send event interrupt for CM4
	CM4_SEV_IRQHandler,					// 65	CM4 Send event interrupt for CM7
	0,									// 66
	0,									// 67
#endif
	DMA2_Stream5_IRQHandler,			// 68	DMA2 Stream 5 global interrupt
	DMA2_Stream6_IRQHandler,			// 69	DMA2 Stream 6 global interrupt
	DMA2_Stream7_IRQHandler,			// 70	DMA2 Stream 7 global interrupt
	USART6_IRQHandler,					// 71	USART6 global interrupt
	I2C3_EV_IRQHandler,					// 72	I2C3 event interrupt
	I2C3_ER_IRQHandler,					// 73	I2C3 error interrupt
	OTG_HS_EP1_OUT_IRQHandler,			// 74	USB OTG HS End Point 1 Out global interrupt
	OTG_HS_EP1_IN_IRQHandler,			// 75	USB OTG HS End Point 1 In global interrupt
	OTG_HS_WKUP_IRQHandler,				// 76	USB OTG HS Wakeup through EXTI interrupt
	OTG_HS_IRQHandler,					// 77	USB OTG HS global interrupt
	DCMI_IRQHandler,					// 78	DCMI
	CRYP_IRQHandler,					// 79	Crypto global interrupt
	HASH_RNG_IRQHandler,				// 80	Hash and/or Rng global interrupt
	FPU_IRQHandler,						// 81	FPU
	UART7_IRQHandler,					// 82	UART7
	UART8_IRQHandler,					// 83	UART8
	SPI4_IRQHandler,					// 84	SPI4
	SPI5_IRQHandler,					// 85	SPI5
	SPI6_IRQHandler,					// 86	SPI6
	SAI1_IRQHandler,					// 87	SAI1
	LTDC_IRQHandler,					// 88	LTDC
	LTDC_ER_IRQHandler,					// 89	LTDC error
	DMA2D_IRQHandler,					// 90	DMA2D
	SAI2_IRQHandler,					// 91	SAI2
	QUADSPI_IRQHandler,					// 92	QUADSPI
	LPTIM1_IRQHandler,					// 93	LPTIM1
	CEC_IRQHandler,						// 94	HDMI_CEC
	I2C4_EV_IRQHandler,					// 95	I2C4 Event
	I2C4_ER_IRQHandler,					// 96	I2C4 Error
	SPDIF_RX_IRQHandler,				// 97	SPDIF_RX
	OTG_FS_EP1_OUT_IRQHandler,			// 98	USB OTG FS End Point 1 Out
	OTG_FS_EP1_IN_IRQHandler,			// 99	USB OTG FS End Point 1 In
	OTG_FS_WKUP_IRQHandler,				// 100	USB OTG FS Wakeup through
	OTG_FS_IRQHandler,					// 101	USB OTG FS
	DMAMUX1_OVR_IRQHandler,				// 102	DMAMUX1 Overrun interrupt
	HRTIM1_Master_IRQHandler,			// 103	HRTIM Master Timer global Interrupt
	HRTIM1_TIMA_IRQHandler,				// 104	HRTIM Timer A global Interrupt
	HRTIM1_TIMB_IRQHandler,				// 105	HRTIM Timer B global Interrupt
	HRTIM1_TIMC_IRQHandler,				// 106	HRTIM Timer C global Interrupt
	HRTIM1_TIMD_IRQHandler,				// 107	HRTIM Timer D global Interrupt
	HRTIM1_TIME_IRQHandler,				// 108	HRTIM Timer E global Interrupt
	HRTIM1_FLT_IRQHandler,				// 109	HRTIM Fault global Interrupt
	DFSDM1_FLT0_IRQHandler,				// 110	DFSDM Filter0 Interrupt
	DFSDM1_FLT1_IRQHandler,				// 111	DFSDM Filter1 Interrupt
	DFSDM1_FLT2_IRQHandler,				// 112	DFSDM Filter2 Interrupt
	DFSDM1_FLT3_IRQHandler,				// 113	DFSDM Filter3 Interrupt
	SAI3_IRQHandler,					// 114	SAI3 global Interrupt
	SWPMI1_IRQHandler,					// 115	Serial Wire Interface 1 global interrupt
	TIM15_IRQHandler,					// 116	TIM15 global Interrupt
	TIM16_IRQHandler,					// 117	TIM16 global Interrupt
	TIM17_IRQHandler,					// 118	TIM17 global Interrupt
	MDIOS_WKUP_IRQHandler,				// 119	MDIOS Wakeup  Interrupt
	MDIOS_IRQHandler,					// 120	MDIOS global Interrupt
	JPEG_IRQHandler,					// 121	JPEG global Interrupt
	MDMA_IRQHandler,					// 122	MDMA global Interrupt
	DSI_IRQHandler,						// 123	DSI global Interrupt
	SDMMC2_IRQHandler,					// 124	SDMMC2 global Interrupt
	HSEM1_IRQHandler,					// 125	HSEM1 global Interrupt
	HSEM2_IRQHandler,					// 126	HSEM2 global Interrupt
#if (defined DAC2)
	DAC2_IRQHandler,					// 127	DAC2 global Interrupt
#else
	ADC3_IRQHandler,					// 127	ADC3 global Interrupt
#endif
	DMAMUX2_OVR_IRQHandler,				// 128	DMAMUX Overrun interrupt
	BDMA_Channel0_IRQHandler,			// 129	BDMA Channel 0 global Interrupt
	BDMA_Channel1_IRQHandler,			// 130	BDMA Channel 1 global Interrupt
	BDMA_Channel2_IRQHandler,			// 131	BDMA Channel 2 global Interrupt
	BDMA_Channel3_IRQHandler,			// 132	BDMA Channel 3 global Interrupt
	BDMA_Channel4_IRQHandler,			// 133	BDMA Channel 4 global Interrupt
	BDMA_Channel5_IRQHandler,			// 134	BDMA Channel 5 global Interrupt
	BDMA_Channel6_IRQHandler,			// 135	BDMA Channel 6 global Interrupt
	BDMA_Channel7_IRQHandler,			// 136	BDMA Channel 7 global Interrupt
	COMP1_IRQHandler,					// 137	COMP1 and COMP2 global Interrupt
	LPTIM2_IRQHandler,					// 138	LP TIM2 global interrupt
	LPTIM3_IRQHandler,					// 139	LP TIM3 global interrupt
	LPTIM4_IRQHandler,					// 140	LP TIM4 global interrupt
	LPTIM5_IRQHandler,					// 141	LP TIM5 global interrupt
	LPUART1_IRQHandler,					// 142	LP UART1 interrupt
	WWDG_RST_IRQHandler,				// 143	Window Watchdog reset interrupt (exti_d2_wwdg_it, exti_d1_wwdg_it
	CRS_IRQHandler,						// 144	Clock Recovery Global Interrupt
	ECC_IRQHandler,						// 145	ECC diagnostic global interrupt
	SAI4_IRQHandler,					// 146	SAI4 global interrupt
	DTS_IRQHandler,						// 147	DTS
	HOLD_CORE_IRQHandler,				// 148	Hold core interrupt
	WAKEUP_PIN_IRQHandler,				// 149	Interrupt for all 6 wake-up pins
	OCTOSPI2_IRQHandler,				// 150	OCTOSPI2
	OTFDEC1_IRQHandler,					// 151	OTFDEC1
	OTFDEC2_IRQHandler,					// 152	OTFDEC2
#if (defined GFXMMU)
	GFXMMU_IRQHandler,					// 153	GFXMMU
	BDMA1_IRQHandler,					// 154	BDMA1
#else
	FMAC_IRQHandler,					// 153
	CORDIC_IRQHandler,					// 154
#endif
	UART9_IRQHandler,					// 155
	USART10_IRQHandler,					// 156
	I2C5_EV_IRQHandler,					// 157
	I2C5_ER_IRQHandler,					// 158
	FDCAN3_IT0_IRQHandler,				// 159	FDCAN3 Interrupt line 0
	FDCAN3_IT1_IRQHandler,				// 160	FDCAN3 Interrupt line 1
	TIM23_IRQHandler,					// 161
	TIM24_IRQHandler,					// 162
} ;

//-----------------------------------------------------------------------------
// The processor jumps here if an unexpected interrupt occurs or a
// specific handler is not present in the application code.
// When in DEBUG, triggers a debug exception to warn the user

void __attribute__ ((section(".isr_handlers"))) Default_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Default exception handlers. Override the ones here by defining your own
// handler routines in your application code.
// When in DEBUG, triggers a debug exception to warn the user

void __attribute__ ((section(".isr_handlers"), noreturn, naked)) Reset_Handler (void)
{
	start_ () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) NMI_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) SVC_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) PendSV_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// Hard Fault handler wrapper in assembly.
// It extracts the location of stack frame and passes it to handler in C as a pointer.
// We also pass the LR value as second parameter.
// (Based on Joseph Yiu's, The Definitive Guide to ARM Cortex-M3 and
// Cortex-M4 Processors, Third Edition, Chap. 12.8, page 402).

// naked attribute:
// Use this attribute on the ARM ports to indicate that the specified function
// does not need prologue/epilogue sequences generated by the compiler.
// It is up to the programmer to provide these sequences.
// The only statements that can be safely included in naked functions are asm statements that do not have operands.
// All other statements, including declarations of local variables, if statements, and so forth, should be avoided.
// Naked functions should be used to implement the body of an assembly function,
// while allowing the compiler to construct the requisite function declaration for the assembler.

void __attribute__ ((section(".isr_handlers"),weak,naked)) HardFault_Handler (void)
{
	__ASM volatile
	(
		" tst	lr,#4	\n"
		" ite	eq		\n"
		" mrseq	r0,msp	\n"
		" mrsne	r0,psp	\n"
		" mov	r1,lr	\n"
		" ldr	r2,=HardFault_Handler_C \n"
		" bx	r2"

		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) HardFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr  ;
	#endif

	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Exceptions specific to ARM ARCH 7M /  7EM

#if defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

void __attribute__ ((section(".isr_handlers"),weak)) MemManage_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak)) DebugMon_Handler (void)
{
	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak,naked)) BusFault_Handler (void)
{
	__ASM volatile
	(
		" tst		lr,#4	\n"
		" ite		eq		\n"
		" mrseq		r0,msp	\n"
		" mrsne		r0,psp	\n"
		" mov		r1,lr	\n"
		" ldr		r2,=BusFault_Handler_C \n"
		" bx		r2"
		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) BusFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr  ;
	#endif

	__DEBUG_BKPT () ;
}

// ----------------------------------------------------------------------------

void __attribute__ ((section(".isr_handlers"),weak,naked)) UsageFault_Handler (void)
{
	__ASM volatile
	(
		" tst		lr,#4	\n"
		" ite		eq		\n"
		" mrseq		r0,msp	\n"
		" mrsne		r0,psp	\n"
		" mov		r1,lr	\n"
		" ldr		r2,=UsageFault_Handler_C \n"
		" bx		r2"
		: // Outputs
		: // Inputs
		: // Clobbers
	) ;
}

// Only referenced by inline assembly, so need "used" attribute
// The user may redefine this function, so "weak" attribute
void __attribute__ ((section(".isr_handlers"),weak,used)) UsageFault_Handler_C (
						exceptionStackFrame		* frame,
						uint32_t 				lr)
{
	(void) frame ;
	(void) lr ;

	#if defined (DEBUG)
		uint32_t	mmfar	= SCB->MMFAR ;	// MemManage Fault Address
		uint32_t	bfar 	= SCB->BFAR ;	// Bus Fault Address
		uint32_t	cfsr 	= SCB->CFSR ;	// Configurable Fault Status Registers

		(void) mmfar;
		(void) bfar ;
		(void) cfsr ;
	#endif

	__DEBUG_BKPT () ;
}

#endif	// defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__)

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

