/*
----------------------------------------------------------------------

	AdAstra - Real Time Kernel

	Alain Chebrou

	dacbasic.h	DAC management with DMA

	When		Who	What
	05/05/18	ac	Creation for STM32F4
	14/03/20	ac	Port to STM32H7xx and STM32L4+
	26/10/21	ac	Port to STM32G0
					Conditionally defined DAC_DMA_DOUBLE
	04/06/22	ac	Add VREFBUF setting
	12/08/22	ac	dacCap_t and dacIoDesc_t made public so user can write more optimized code

----------------------------------------------------------------------
*/
/*
----------------------------------------------------------------------
	Copyright (c) 2013-2023 Alain Chebrou - AdAstra-Soft . com
	All rights reserved.

	This file is part of the AdAstra Real Time Kernel distribution.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	- Redistributions of source code must retain the above copyright
	  notice, this list of conditions and the following disclaimer.

	- Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.

	- Neither the name of the copyright holder nor the
	  names of its contributors may be used to endorse or promote products
	  derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
----------------------------------------------------------------------
*/

#if ! defined DACBASIC_H_
#define DACBASIC_H_
//----------------------------------------------------------------------

#include	"dmabasic.h"
#include	"gpiobasic.h"

#if (defined DAC1)		// Some devices like STM30G070 doesn't have DAC
//----------------------------------------------------------------------

#if (defined STM32G0)
	#include	"stm32g0xx_ll_dac.h"	// For LL_DAC_TRIG_xx definitions
#endif

#if (defined STM32G4)
	#include	"stm32g4xx_ll_dac.h"	// For LL_DAC_TRIG_xx definitions
#endif

#if (defined STM32L4)
	#include	"stm32l4xx_ll_dac.h"	// For LL_DAC_TRIG_xx definitions
#endif

#if (defined STM32H7)
	#include	"stm32h7xx_ll_dac.h"	// For LL_DAC_TRIG_xx definitions
#endif

// To define the supported DAC devices

#define	WITH_DAC_1		1	// DAC1 with 2 channels
#define	WITH_DAC_2		0	// WARNING: DAC2 is not supported by this implementation

// If one WITH_DAC_x is 1 then dacDescArray in dacbasic.c is not empty and must be defined
#if (WITH_DAC_1 == 1  ||  WITH_DAC_2 == 1)
	#define	WITH_DAC_X		1
#endif

//----------------------------------------------------------------------
// Initialization and setting flags

#define		DAC_CHAN1			0x00000001		// Channel 1 id
#define		DAC_CHAN2			0x00000002		// Channel 2 id

// For dacInit()
#define		DAC_MONO1			0x00000010		// Use independent channel 1
#define		DAC_MONO2			0x00000020		// Use independent channel 2
#define		DAC_DUAL			0x00000040		// Use dual channel 1 and 2

// For dacSetChannel()
#define		DAC_MODEMASK		0x00000700		// (internal use)
#define		DAC_OUTBUFFDIS		0x00000200		// Disable output buffer
#define		DAC_OUTBUFFEN		0x00000000		// Enable  output buffer (default)

#define		DAC_ONCHIPDIS		0x00000000		// Disable connection to on chip peripherals
#define		DAC_ONSHIPEN		0x00000100		// Enable  connection to on chip peripherals

#define		DAC_HOLDEN			0x00000400		// DAC channel in hold mode
#define		DAC_HOLDDIS			0x00000000		// DAC channel in normal mode

#define		DAC_BITS8			0x00001000		// 8 bits words
#define		DAC_BITS12			0x00000000		// 12 bits words (default)

#define		DAC_ALIGNLEFT		0x00002000		// Left aligned data
#define		DAC_ALIGNRIGHT		0x00000000		// Right aligned data (default)

// Instead of using DAC_OUTxx + DAC_ONxx + DAC_HOLDxx, you can use these clearer shortcuts
#define		DAC_MODE_NORM_BUF_EXT		(0u)											// DAC is connected to the external pin
#define		DAC_MODE_NORM_BUF_EXT_INT	(DAC_ONSHIPEN)									// DAC is connected to external pin and to on-chip peripherals
#define		DAC_MODE_NORM_NOBUF_EXT		(DAC_OUTBUFFDIS)								// DAC is connected to the external pin
#define		DAC_MODE_NORM_NOBUF_INT		(DAC_OUTBUFFDIS | DAC_ONSHIPEN)					// DAC is connected to on-chip peripherals

#define		DAC_MODE_HOLD_BUF_EXT		(DAC_HOLDEN)									// DAC is connected to the external pin
#define		DAC_MODE_HOLD_BUF_EXT_INT	(DAC_HOLDEN | DAC_ONSHIPEN)						// DAC is connected to external pin and to on chip peripherals
#define		DAC_MODE_HOLD_NOBUF_EXT_INT	(DAC_HOLDEN | DAC_OUTBUFFDIS)					// DAC is connected to external pin and to on chip peripherals
#define		DAC_MODE_HOLD_NOBUF_INT		(DAC_HOLDEN | DAC_ONSHIPEN | DAC_OUTBUFFDIS)	// DAC is connected to on chip peripherals

// For dacDmaInit()
#define		DAC_DMA_MASK		0x00070000		// (Internal use)
#define		DAC_DMA_CIRCULAR	0x00010000		// Circular DMA buffer
#if (defined DMA_SxCR_DBM)
#define		DAC_DMA_DOUBLE		0x00020000		// Double DMA buffer
#endif
#define		DAC_DMA_SINGLE		0x00040000		// One DMA buffer, not circular

// Mode for dacSetWave()
#define		DAC_GEN_NONE		0x00000000		// Generation disabled
#define		DAC_GEN_NOISE		0x00000001		// Noise generation mode
#define		DAC_GEN_WAVE		0x00000002		// Triangle wave generation mode

// Other flags
#define		DAC_DMA_NOIT		0x00100000		// No DAC/DMA interrupt
#define		DAC_TRIGGER_NONE	0x00200000		// trigScr field of dacSetChannel() when no trigger source

// The DAC handle returned by dacInit()
typedef void 	* dacHandle_t ;

// Prototype of DAC callback function
typedef void (* dacCallback_t) (dacHandle_t hdl, uint32_t event, uint32_t channel, uintptr_t arg) ;

//----------------------------------------------------------------------

typedef struct dacDesc_s
{
	uint16_t		id ;			// Descriptor id
	DAC_TypeDef		* pDac ;		// Pointer to DAC registers
	dmaDesc_t		dma1 ;
	dmaDesc_t		dma2 ;

} dacDesc_t ;

//----------------------------------------------------------------------
//	DAC I/O management descriptors, intended for dacbasic internal use.
//	It is public so the user can write more optimized functions

#if (defined STM32H7)
typedef struct
{
	volatile	uint32_t		* rccENR ;			// @ of register to enable DAC clock
	volatile	uint32_t		* rccRSTR ;			// @ of register to reset the DAC peripheral
				uint32_t		periphMask ;		// Bit mask to use with RCC registers
				IRQn_Type		irqVector ;			// DMA interrupt vector
				uint16_t		chanReqNum [2] ;	// DMA request numbers
				gpioPinDesc_t	dacOut     [2] ;	// DAC channel 1 and 2 output pins descriptor

} dacCap_t ;

typedef struct
{
	const dacDesc_t	* pDacDesc ;		// DAC descriptor
	const dacCap_t	* pDacCap ;			// DAC capabilities
	uint32_t		flags ;				// Common configuration flags
	uint32_t		flagsChan [2] ;		// Channels configuration flags
	uint32_t		dataReg   [2] ;		// DAC data register address (DAC_DHxx)
	uint32_t		dmaSize   [2] ;		// DMA buffer size in bytes, for cache management
	uint32_t		dmaLength [2] ;		// DMA buffer size in items, for NDTR
	dacCallback_t	cb        [2] ;		// DAC interrupt callback
	uintptr_t		cbArg     [2] ; 	// DAC interrupt callback arg

} dacIoDesc_t ;

#else

typedef struct
{
	uint32_t		periphMask ;		// Bit mask to use with RCC registers
	IRQn_Type		irqVector ;			// DMA interrupt vector
	uint16_t		chanReqNum [2] ;	// DMA request numbers for DMAMUX
	gpioPinDesc_t	dacOut     [2] ;	// DAC channel 1 and 2 output pins descriptor

} dacCap_t ;

typedef struct
{
	const dacDesc_t	* pDacDesc ;		// DAC descriptor
	const dacCap_t	* pDacCap ;			// DAC capabilities
	uint32_t		flags ;				// Common configuration flags
	uint32_t		flagsChan [2] ;		// Channels configuration flags
	uint32_t		dataReg   [2] ;		// DAC data register address (DAC_DHxx)
	uint32_t		dmaLength [2] ;		// DMA buffer size in items, for NDTR
	dacCallback_t	cb        [2] ;		// DAC interrupt callback
	uintptr_t		cbArg     [2] ; 	// DAC interrupt callback arg

} dacIoDesc_t ;
#endif

//----------------------------------------------------------------------

//	Default priority for DAC interrupt
#define		DAC_INT_PRIORITY	BSP_IRQPRIOMAX_MINUS(2)

#define		DAC_EV_UNDR			0x10000		// Underrun event for DAC interrupt callback

typedef enum
{
	DAC_ENONE		= 0,
	DAC_EFAIL		= 1,
	DAC_ENOTINIT	= 2,
	DAC_ETIMEOUT	= 3,
	DAC_EARG		= 4,
	DAC_ESTATE		= 5

} dacStatus_t ;

#ifdef __cplusplus
extern "C" {
#endif

dacStatus_t		dacGetDesc 			(uint32_t id, const dacDesc_t ** ppDacDesc) ;
dacStatus_t 	dacInit				(const dacDesc_t * pDacDesc, uint32_t flags, dacHandle_t * pHdl) ;
dacStatus_t 	dacSetChannel		(dacHandle_t hdl, uint32_t channel, uint32_t flags, uint32_t trigSrc) ;
dacStatus_t 	dacSetCallback		(dacHandle_t hdl, uint32_t channel, dacCallback_t cb, uintptr_t arg) ;
dacStatus_t 	dacDmaInit			(dacHandle_t hdl, uint32_t channel, uint32_t flags, uint32_t length) ;
dacStatus_t 	dacDmaBuffer		(dacHandle_t hdl, uint32_t channel, const void * buffer1, const void * buffer2) ;
dacStatus_t 	dacStart			(dacHandle_t hdl, uint32_t channel) ;
dacStatus_t 	dacStop				(dacHandle_t hdl, uint32_t channel) ;
dacStatus_t 	dacSwTrigger		(dacHandle_t hdl, uint32_t channel) ;
dacStatus_t 	dacOuput			(dacHandle_t hdl, uint32_t channel, uint32_t value) ;

// To set the DAC output in Sample / hold mode
dacStatus_t 	dacSetHold			(dacHandle_t hdl, uint32_t channel, uint32_t sample, uint32_t hold, uint32_t refresh) ;

// For channel calibration
dacStatus_t		dacSetCalibration	(dacHandle_t hdl, uint32_t channel, uint32_t bEnable) ;
dacStatus_t		dacSetOtrim			(dacHandle_t hdl, uint32_t channel, uint32_t otrim) ;
dacStatus_t		dacGetOtrim			(dacHandle_t hdl, uint32_t channel, uint32_t * pOtrim) ;
dacStatus_t		dacGetCalFlag		(dacHandle_t hdl, uint32_t channel, uint32_t delay, uint32_t * pFlag) ;
dacStatus_t		dacCalibration		(dacHandle_t hdl, uint32_t channel, uint32_t delay) ;

// For noise or triangle generation
dacStatus_t		dacSetWave			(dacHandle_t hdl, uint32_t channel, uint32_t mode, uint32_t MAMP) ;

//----------------------------------------------------------------------
#if (defined VREFBUF)

#define	VREFBUF_MODE_OFF	0u	//	VREFBUF buffer off:
								//	VREF+ pin pulled-down to VSSA

#define	VREFBUF_MODE_EXT	1u	//	External voltage reference mode (default value):
								//	VREFBUF buffer off
								//	VREF+ pin input mode

#define	VREFBUF_MODE_INT	2u	//	Internal voltage reference mode:
								//	VREFBUF buffer ON
								//	VREF+ pin connected to VREFBUF buffer output

#define	VREFBUF_MODE_HOLD	3u	//	Hold mode:
								//	VREFBUF buffer off
								//	VREF+ pin floating. The voltage is held with the external capacitor
								//	VRR detection disabled and VRR bit keeps last state

#if (defined STM32H7)
#define	VREFBUF_SCALE_2_5	0u	//	VREF_OUT1 : 2.5V
#define	VREFBUF_SCALE_2_048	1u	//	VREF_OUT2 : 2.048V
#define	VREFBUF_SCALE_1_8	2u	//	VREF_OUT3 : 1.8V
#define	VREFBUF_SCALE_1_5	3u	//	VREF_OUT4 : 1.5V

#elif (defined STM32G0 || defined STM32L4)
#define	VREFBUF_SCALE0		0u	//	VREF_OUT1 : 2.048V
#define	VREFBUF_SCALE1		1u	//	VREF_OUT2 : 2.5V

#elif (defined STM32G4)
#define	VREFBUF_SCALE_2_048	0u	//	VREF_OUT2 : 2.048V
#define	VREFBUF_SCALE_2_5	1u	//	VREF_OUT1 : 2.5V
#define	VREFBUF_SCALE_2_9	2u	//	VREF_OUT3 : 2_9V
#endif

// Set VREFBUF mode and scale
void	vrefbufSet (uint32_t vrefbufMode, uint32_t scale) ;

// Set the analog reference buffer trim value
__ALWAYS_STATIC_INLINE void	vrefbufSetTrim (uint32_t trim)
{
	VREFBUF->CCR = trim & VREFBUF_CCR_TRIM ;
}

// Get the analog reference buffer trim value
__ALWAYS_STATIC_INLINE uint32_t	vrefbufGetTrim (void)
{
	return VREFBUF->CCR & VREFBUF_CCR_TRIM ;
}

#endif	// VREFBUF
//----------------------------------------------------------------------


#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
#endif	// DAC1
#endif	// DACBASIC_H_
