/*
 * Linker script for Cortex-M - STM32H743
 * This file is included by the specifig MCU linker script
 * 
 * Use multi-region initializations for data and bss
 */

 /*
	To set a function in specific RAM section:
	__attribute__ ((section(".text_RAM_D2")))   void ram2Function (void) 
	__attribute__ ((section(".text_ITCM")))     void itcmFunction (void) 
	
	To set an initialized data in a DTCM or RAM:
	__attribute__ ((section(".data_DTCM")))	    int32_t	nbData = 3 ;
	__attribute__ ((section(".data_RAM_D2")))	int32_t	nbData = 3 ;
	__attribute__ ((section(".data_RAM_D3")))	int32_t	nbData = 3 ;
	__attribute__ ((section(".data_BKPRAM")))	int32_t	nbData = 3 ;
	
	To set a zeroized data in DTCM or RAM:
	__attribute__ ((section(".bss_DTCM")))	    int32_t	nbData ;
	__attribute__ ((section(".bss_RAM_D2")))	int32_t	nbData ;
	__attribute__ ((section(".bss_RAM_D3")))	int32_t	nbData ;
	__attribute__ ((section(".bss_BKPRAM")))	int32_t	nbData ;

	To set an uninitialized data in DTCM or RAM:
	__attribute__ ((section(".noinit_DTCM")))	int32_t	nbData ;
	__attribute__ ((section(".noinit_RAM_D2")))	int32_t	nbData ;
	__attribute__ ((section(".noinit_RAM_D3")))	int32_t	nbData ;
	__attribute__ ((section(".noinit_BKPRAM")))	int32_t	nbData ;

	There is no provision to put data or bss section in external RAM
*/

/*--------------------------------------------------------------------------
/* The main (privileged) stack definition, default (minimum) heap definition */

/* Required amount of RAM for main stack */
Main_Stack_Size_ = 1024 ;

_estack_ = ORIGIN(RAM_D1) + LENGTH(RAM_D1);		/* End of main stack pointer just beyond the RAM */
_sstack_ = _estack_  - Main_Stack_Size_ ;		/* Start of main stack	*/

/* Default heap definitions.
 * The heap start immediately after the last statically allocated 
 * .sbss/.noinit section, and extends up to the main stack start.
 */
 Min_Heap_Size_ = 1024 ;					/* For RAM size check only */
PROVIDE ( _heap_begin_ = _end_noinit ) ;
PROVIDE ( _heap_end_   = _sstack_ ) ;

/* The entry point is informative since the Cortex-M reset vector points to it anyway */
ENTRY(start_)


/*--------------------------------------------------------------------------
 * Sections Definitions
 */

SECTIONS
{
    /*
     * The main stack address, IRQ vectors, the default IRQ handlers and the startup code
     * are stored in the .isr_vectors section, which goes to beginning of FLASH. 
     */
    .isr_vectors : ALIGN(4)
    {
        FILL(0xFF)
        
        KEEP(*(.isr_stackaddress))			/* Interrupt stack address */
        KEEP(*(.isr_vectors))				/* Interrupt vectors */
        *(.isr_handlers .isr_handlers.*)	/* Interrupt handlers */
        *(.startup .startup.*)				/* Startup code*/

   } >FLASH

    .inits : ALIGN (4)
    {
        /* 
         * Memory regions initialization tables.
         *
         * There are two kinds of arrays for each RAM region: one for data and one for bss.
         * The startup code uses these tables to perform the initialization.
         * 
         * The data table includes:
         * - from (LOADADDR()) (in FLASH)
         * - region_begin (ADDR())
         * - region_end (ADDR()+SIZEOF())
         *
         * The bss table includes:
         * - region_begin (ADDR())
         * - region_end (ADDR()+SIZEOF())
         *
         * WARNING: It is mandatory that the regions are word aligned, 
         * since the initialization code works only on words.
         */

        __data_regions_array_start = . ;

		/* ITCM code */
        LONG(LOADADDR(.text_ITCM)) ;
        LONG(ADDR(.text_ITCM)) ;
        LONG(ADDR(.text_ITCM)+SIZEOF(.text_ITCM)) ;

		/* Main RAM initialized data */
        LONG(LOADADDR(.data)) ;
        LONG(ADDR(.data)) ;
        LONG(ADDR(.data)+SIZEOF(.data)) ;

		/* RAM_D2 initialized data */
        LONG(LOADADDR(.data_RAM_D2)) ;
        LONG(ADDR(.data_RAM_D2)) ;
        LONG(ADDR(.data_RAM_D2)+SIZEOF(.data_RAM_D2)) ;

		/* RAM_D3 initialized data */
        LONG(LOADADDR(.data_RAM_D3)) ;
        LONG(ADDR(.data_RAM_D3)) ;
        LONG(ADDR(.data_RAM_D3)+SIZEOF(.data_RAM_D3)) ;

		/* DTCM initialized data */
        LONG(LOADADDR(.data_DTCM)) ;
        LONG(ADDR(.data_DTCM)) ;
        LONG(ADDR(.data_DTCM)+SIZEOF(.data_DTCM)) ;

		/* Backup RAM initialized data */
        LONG(LOADADDR(.data_BKPRAM)) ;
        LONG(ADDR(.data_BKPRAM)) ;
        LONG(ADDR(.data_BKPRAM)+SIZEOF(.data_BKPRAM)) ;

        __data_regions_array_end = . ;

        __bss_regions_array_start = . ;

        LONG(ADDR(.bss)) ;
        LONG(ADDR(.bss)+SIZEOF(.bss)) ;

        LONG(ADDR(.bss_DTCM)) ;
        LONG(ADDR(.bss_DTCM)+SIZEOF(.bss_DTCM)) ;

        LONG(ADDR(.bss_RAM_D2)) ;
        LONG(ADDR(.bss_RAM_D2)+SIZEOF(.bss_RAM_D2)) ;

        LONG(ADDR(.bss_RAM_D3)) ;
        LONG(ADDR(.bss_RAM_D3)+SIZEOF(.bss_RAM_D3)) ;

        LONG(ADDR(.bss_BKPRAM)) ;
        LONG(ADDR(.bss_BKPRAM)+SIZEOF(.bss_BKPRAM)) ;

        __bss_regions_array_end = . ;
   
		/*--------------------------------------------------------------------------
         * The preinit code, i.e. an array of pointers to initialization 
         * functions to be performed before constructors.
         */
		. = ALIGN (4) ;
		PROVIDE_HIDDEN (__preinit_array_start = .) ;
        
        /* The application inits. If you need to enforce some order in 
         * execution, create new sections before this.
         */
		KEEP (*(.preinit_array .preinit_array.*))

		PROVIDE_HIDDEN (__preinit_array_end = .) ;

		/*--------------------------------------------------------------------------
         * The init code, i.e. an array of pointers to static constructors.
         */
		. = ALIGN (4) ;
		PROVIDE_HIDDEN (__init_array_start = .) ;
		KEEP (*(SORT(.init_array.*)))
		KEEP (*(.init_array))
		PROVIDE_HIDDEN (__init_array_end = .) ;

		. = ALIGN (4) ;

		/*
         * The fini code, i.e. an array of pointers to static destructors.
         */
		PROVIDE_HIDDEN (__fini_array_start = .) ;
		KEEP (*(SORT(.fini_array.*)))
		KEEP (*(.fini_array))
		PROVIDE_HIDDEN (__fini_array_end = .) ;

    } >FLASH

    /*--------------------------------------------------------------------------
     * The program code is stored in the .text section, which goes to FLASH.
     */
    .text : ALIGN(4)
    {
        *(.text .text.*)			/* all remaining code */
 
 		/* read-only data (constants) */
        *(.rodata .rodata.* .constdata .constdata.*) 		

        *(vtable)					/* C++ virtual tables */

		KEEP(*(.eh_frame*))

        *(.glue_7)         /* glue arm to thumb code */
        *(.glue_7t)        /* glue thumb to arm code */

    } >FLASH

    .text_ITCM : ALIGN(4)
    {
       FILL(0xFF)
       *(.text_ITCM .text_ITCM.*)
       . = ALIGN(4) ;
    } >ITCM AT>FLASH

	/* ARM magic sections */
	.ARM.extab : ALIGN(4)
   	{
       *(.ARM.extab* .gnu.linkonce.armextab.*)
   	} > FLASH

    . = ALIGN(4);
   	.ARM.exidx : ALIGN(4)
   	{
       __exidx_start = .;   	
       *(.ARM.exidx* .gnu.linkonce.armexidx.*)
       __exidx_end = .;
   	} > FLASH
   	
    . = ALIGN(4);
    _etext = .;
    __etext = .;
    
    /* MEMORY_ARRAY */
/*
    .ROarraySection :
    {
     	*(.ROarraySection .ROarraySection.*)                          
    } >MEMORY_ARRAY
*/

	/* --------------------------------------------------------------------------
     * The initialized data sections.
     *
     * The program executes knowing that the data is in the RAM
     * but the compiler puts the initial values in the FLASH (inidata).
     * It is one task of the startup to copy the initial values from 
     * FLASH to RAM.
     */
    .data : ALIGN(4)
    {
    	FILL(0xFF)

        _sdata = . ;
		*(.data_begin .data_begin.*)

		*(.data .data.*)
		
		*(.data_end .data_end.*)
	    . = ALIGN(4);

	    /* This is used by the startup code to initialize the .data section */
        _edata = . ;

    } >RAM_D1 AT>FLASH
    
    .data_RAM_D2 : ALIGN(4)
    {
       FILL(0xFF)
       *(.data_RAM_D2 .data_RAM_D2.*)
       . = ALIGN(4) ;
    } >RAM_D2 AT>FLASH

    .data_RAM_D3 : ALIGN(4)
    {
       FILL(0xFF)
       *(.data_RAM_D3 .data_RAM_D3.*)
       . = ALIGN(4) ;
    } >RAM_D3 AT>FLASH

    .data_DTCM : ALIGN(4)
    {
       FILL(0xFF)
       *(.data_DTCM .data_DTCM.*)
       . = ALIGN(4) ;
    } >DTCM AT>FLASH

    .data_BKPRAM : ALIGN(4)
    {
       FILL(0xFF)
       *(.data_BKPRAM .data_BKPRAM.*)
       . = ALIGN(4) ;
    } >BKPRAM AT>FLASH

    /*--------------------------------------------------------------------------
     * The zeroized data sections. NOLOAD is used to avoid
     * the section type changed to PROGBITS
     */
     
    /* The RAM_D2 uninitialized data section. */
	.bss_RAM_D2 (NOLOAD) : ALIGN(4)
	{
		*(.bss_RAM_D2 .bss_RAM_D2.*)
	} > RAM_D2

    /* The DTCM uninitialized data section. */
	.bss_DTCM (NOLOAD) : ALIGN(4)
	{
		*(.bss_DTCM .bss_DTCM.*)
	} > DTCM

    /* The RAM_D3 uninitialized data section. */
	.bss_RAM_D3 (NOLOAD) : ALIGN(4)
	{
		*(.bss_RAM_D3 .bss_RAM_D3.*)
	} > RAM_D3

    /* The BKPRAM uninitialized data section. */
	.bss_BKPRAM (NOLOAD) : ALIGN(4)
	{
		*(.bss_BKPRAM .bss_BKPRAM.*)
	} > BKPRAM

    /* The primary zeroized data section. */
    .bss (NOLOAD) : ALIGN(4)
    {
        __bss_start__ = .;     	/* standard newlib definition */
        *(.bss_begin .bss_begin.*)

        *(.bss .bss.*)
        *(COMMON)
        
        *(.bss_end .bss_end.*)
	    . = ALIGN(4);
        __bss_end__ = .;        /* standard newlib definition */

    } >RAM_D1

    /*--------------------------------------------------------------------------
     * The uninitialized data sections. NOLOAD is used to avoid
     * the section type changed to PROGBITS
     */
    .noinit_RAM_D2 (NOLOAD) : ALIGN(4)
    {
        *(.noinit_RAM_D2 .noinit_RAM_D2.*)         
         . = ALIGN(4) ;
    } > RAM_D2

    .noinit_RAM_D3 (NOLOAD) : ALIGN(4)
    {
        *(.noinit_RAM_D3 .noinit_RAM_D3.*)         
         . = ALIGN(4) ;
    } > RAM_D3

    .noinit_DTCM (NOLOAD) : ALIGN(4)
    {
        *(.noinit_DTCM .noinit_DTCM.*)         
         . = ALIGN(4) ;
    } > DTCM

    .noinit_BKPRAM (NOLOAD) : ALIGN(4)
    {
        *(.noinit_BKPRAM .noinit_BKPRAM.*)         
         . = ALIGN(4) ;
    } > BKPRAM

    /* this section MUST be the last one */
    .noinit (NOLOAD) : ALIGN(4)
    {
        _noinit = .;
        
        *(.noinit .noinit.*) 
        
         . = ALIGN(4) ;
        _end_noinit = .;   
    } > RAM_D1
    
    /* Mandatory to be word aligned, _sbrk assumes this */
    PROVIDE ( end = _end_noinit );
    PROVIDE ( _end = _end_noinit );
    PROVIDE ( __end = _end_noinit );
    PROVIDE ( __end__ = _end_noinit );
    
    /*--------------------------------------------------------------------------
     * Used for validation only, do not allocate anything here!
     *
     * This is to check that there is enough RAM left for the heap and Main stack.
     * It should generate an error if it's full.
     */
    ._check_heap_stack : ALIGN(4)
    {
        . = . + Main_Stack_Size_ + Min_Heap_Size_ ;
    } >RAM_D1
    
    /*--------------------------------------------------------------------------
     * The FLASH Bank2.
     * The C or assembly source must explicitly place the code 
     * or data there using the "section" attribute.
     */
    .b2text : ALIGN(4)
    {
        *(.b2text   .b2text.*)       /* remaining code */
        *(.b2rodata .b2rodata.*)     /* read-only data (constants) */
    } >FLASHB2
    
    /*--------------------------------------------------------------------------
     * The EXTMEM.
     * The C or assembly source must explicitly place the code or data there
     * using the "section" attribute.
     */

    /* EXTMEM Bank0 */
    .eb0text : ALIGN(4)
    {
        *(.eb0text   .eb0text.*)     /* remaining code */
        *(.eb0rodata .eb0rodata.*)   /* read-only data (constants) */
    } >EXTMEMB0
    
    /* EXTMEM Bank1 */
    .eb1text : ALIGN(4)
    {
        *(.eb1text   .eb1text.*)     /* remaining code */
        *(.eb1rodata .eb1rodata.*)   /* read-only data (constants) */
    } >EXTMEMB1
    
    /* EXTMEM Bank2 */
    .eb2text : ALIGN(4)
    {
        *(.eb2text   .eb2text.*)     /* remaining code */
        *(.eb2rodata .eb2rodata.*)   /* read-only data (constants) */
    } >EXTMEMB2
    
    /* EXTMEM Bank3 */
    .eb3text : ALIGN(4)
    {
        *(.eb3text   .eb3text.*)     /* remaining code */
        *(.eb3rodata .eb3rodata.*)   /* read-only data (constants) */
    } >EXTMEMB3
   
	/*--------------------------------------------------------------------------*/
	/*--------------------------------------------------------------------------*/
    /* After that there are only debugging sections. */
    
    /* Remove the debugging information from the standard libraries */    
    DISCARD :
    {
     libc.a ( * )
     libm.a ( * )
     libgcc.a ( * )
    }

  
    /* Stabs debugging sections.  */
    .stab          0 : { *(.stab) }
    .stabstr       0 : { *(.stabstr) }
    .stab.excl     0 : { *(.stab.excl) }
    .stab.exclstr  0 : { *(.stab.exclstr) }
    .stab.index    0 : { *(.stab.index) }
    .stab.indexstr 0 : { *(.stab.indexstr) }
    .comment       0 : { *(.comment) }
 
    /*
     * DWARF debug sections.
     * Symbols in the DWARF debugging sections are relative to the beginning
     * of the section so we begin them at 0.  
     */

    /* DWARF 1 */
    .debug          0 : { *(.debug) }
    .line           0 : { *(.line) }

    /* GNU DWARF 1 extensions */
    .debug_srcinfo  0 : { *(.debug_srcinfo) }
    .debug_sfnames  0 : { *(.debug_sfnames) }

    /* DWARF 1.1 and DWARF 2 */
    .debug_aranges  0 : { *(.debug_aranges) }
    .debug_pubnames 0 : { *(.debug_pubnames) }

    /* DWARF 2 */
    .debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
    .debug_abbrev   0 : { *(.debug_abbrev) }
    .debug_line     0 : { *(.debug_line) }
    .debug_frame    0 : { *(.debug_frame) }
    .debug_str      0 : { *(.debug_str) }
    .debug_loc      0 : { *(.debug_loc) }
    .debug_macinfo  0 : { *(.debug_macinfo) }

    /* SGI/MIPS DWARF 2 extensions */
    .debug_weaknames 0 : { *(.debug_weaknames) }
    .debug_funcnames 0 : { *(.debug_funcnames) }
    .debug_typenames 0 : { *(.debug_typenames) }
    .debug_varnames  0 : { *(.debug_varnames) }    
}
